#ifndef STRINGS_H
#define STRINGS_H

/*
 * Dialog boxes
 */

#define newgame_dialog_str "New Game?"
#define pause_dialog_str "Game paused.  Press Continue to continue."
#define warp_dialog_str "Warp to level?"
#define quit_dialog_str "Quit Game?"

#define story_dialog_str \
"The Story:\n\
\n\
Yet again, the fate of the world rests\n\
in your hands!  An evil computer hacker,\n\
known only by his handle 'Bill', has\n\
created the ultimate computer virus.  A\n\
virus so powerful that it has the power\n\
to transmute an ordinary computer into\n\
a toaster oven.  (oooh!) 'Bill' has\n\
cloned himself into a billion-jillion\n\
micro-Bills.  Their sole purpose is to\n\
deliver the nefarious virus, which has\n\
been cleverly diguised as a popular\n\
operating system.\n\
\n\
As System Administrator/Exterminator,\n\
your job is to keep Bill from succeeding\n\
at his task."

#define rules_dialog_str \
"The Rules:\n\
\n\
xBill has been painstakingly designed and\n\
researched in order to make it as easy to use\n\
for the whole family as it is for little Sally.\n\
Years - nay - days of beta testing and \n\
consulting with the cheapest of human interface\n\
designers have resulted in a game that is easy\n\
to use, yet nothing at all like a Macintosh.\n\
\n\
I.   Whack the Bills (click)\n\
II.  Restart the computer (click)\n\
III. Pick up stolen OSes & return(drag)\n\
     them to their respective computers\n\
IV.  Drag the bucket to extinguish sparks\n\
V.   Scoring is based on total uptime,\n\
     with bonuses for killing Bills.\n\
\n\
As for the rest, you can probably figure\n\
it out.  We did, so it can't be too hard."

#define endgame_dialog_str \
"Module xBill has caused a segmentation fault\n\
at memory address 097E:F1A0.  Core dumped.\n\
\n\
We apologize for the inconvenience."

#define entername_dialog_str "You earned a high score.\nEnter your name:"


/*
 * Menus
 */
#define newgame_menu_str "New Game"
#define pause_menu_str "Pause Game"
#define warp_menu_str "Warp to level..."
#define highscore_menu_str "View High Scores"
#define quit_menu_str "Quit Game"

#define story_menu_str "Story of xBill"
#define rules_menu_str "Rules"
#define about_menu_str "About xBill"

#define score_menu_str "Score"
#define endgame_menu_str "End Game"
#define entername_menu_str "Enter Name"

#endif
